/*
 * ADBMS_Abstraction.h
 *
 *  Created on: 14.07.2022
 *      Author: max
 */

#ifndef INC_ADBMS_ABSTRACTION_H_
#define INC_ADBMS_ABSTRACTION_H_

#include "ADBMS_CMD_MAKROS.h"
#include "ADBMS_LL_Driver.h"
#include "main.h"


#define MAXIMUM_CELL_VOLTAGES 16
#define MAXIMUM_AUX_VOLTAGES  10
#define MAXIMUM_GPIO          10

//see table 103 in datasheet (page 71)
#define DEFAULT_UV 417	//VUV * 16 * 150 uV + 1.5 V     Default Setting 2.5V
#define DEFAULT_OV 1125 //VOV * 16 * 150 uV + 1.5 V		  Default Setting 4.2V

#define mV_from_ADBMS6830(x) (((((int16_t) (x))) * 0.150) + 1500)

struct ADBMS6830_Internal_Status {
  uint16_t CS_FLT : 16; //ADC fault - mismatch between S- and C-ADC
  uint16_t : 3;
  uint16_t CCTS : 13; //Conversion counter
  uint16_t SMED : 1; //S-ADC multiple trim error (uncorrectable)
  uint16_t SED : 1; //S-ADC single trim error (correctable)
  uint16_t CMED : 1; //C-ADC multiple trim error (uncorrectable)
  uint16_t CED : 1; //C-ADC single trim error (correctable)
  uint16_t VD_UV : 1; //3V digital supply undervoltage
  uint16_t VD_OV : 1; //3V digital supply overvoltage
  uint16_t VA_UV : 1; //5V analog supply undervoltage
  uint16_t VA_OV : 1; //5V analog supply overvoltage
  uint16_t OSCCHK : 1; //Oscillator check
  uint16_t TMODCHK : 1; //Test mode check
  uint16_t THSD : 1; //Thermal shutdown
  uint16_t SLEEP : 1; //Sleep mode previously entered
  uint16_t SPIFLT : 1; //SPI fault
  uint16_t COMPARE : 1; //Comparasion between S- and C-ADC active
  uint16_t VDE : 1; //Supply voltage error
  uint16_t VDEL : 1; //Latent supply voltage error
};

typedef struct {
  int16_t cellVoltages[MAXIMUM_CELL_VOLTAGES];
  int16_t auxVoltages[MAXIMUM_AUX_VOLTAGES];

  struct ADBMS6830_Internal_Status status;
  uint16_t internalDieTemp;
  uint16_t analogSupplyVoltage;
  uint16_t digitalSupplyVoltage;
  uint16_t sumOfCellMeasurements;
  uint16_t refVoltage;

  uint16_t GPIO_Values[MAXIMUM_GPIO];

  uint32_t overVoltage;
  uint32_t underVoltage;

} Cell_Module;

uint8_t amsReset();

uint8_t initAMS(SPI_HandleTypeDef* hspi);
uint8_t amsWakeUp();

uint8_t amsCellMeasurement(Cell_Module* module);
uint8_t amsConfigCellMeasurement(uint8_t numberofChannels);

uint8_t amsAuxAndStatusMeasurement(Cell_Module* module);
uint8_t amsConfigAuxMeasurement(uint16_t Channels);

uint8_t amsConfigGPIO(uint16_t gpios);
uint8_t amsSetGPIO(uint16_t gpios);
uint8_t readGPIO(Cell_Module* module);

uint8_t amsConfigBalancing(uint32_t Channels, uint8_t dutyCycle);
uint8_t amsStartBalancing(uint8_t dutyCycle);
uint8_t amsStopBalancing();

uint8_t amsSelfTest();

uint8_t amsConfigOverUnderVoltage(uint16_t overVoltage, uint16_t underVoltage);

uint8_t amsCheckUnderOverVoltage(Cell_Module* module);
uint8_t amsConfigOverVoltage(uint16_t overVoltage);

uint8_t amscheckOpenCellWire(Cell_Module* module);

uint8_t amsClearFlag();
uint8_t amsClearAux();
uint8_t amsClearCells();

uint8_t amsSendWarning();
uint8_t amsSendError();

uint8_t amsClearWarning();
uint8_t amsClearError();

uint8_t amsReadCellVoltages(Cell_Module* module);

#endif /* INC_ADBMS_ABSTRACTION_H_ */
