/*
 * ADBMS_LL_Driver.h
 *
 *  Created on: 05.06.2022
 *      Author: max
 */

#ifndef ADBMS_LL_DRIVER_H_
#define ADBMS_LL_DRIVER_H_

#include "config_ADBMS6830.h"
#include "stm32h7xx_hal.h"
#include <stdint.h>
#define TARGET_STM32


uint8_t adbmsDriverInit(SPI_HandleTypeDef* hspi);

//2 command + 2 PEC + (data + 2 DPEC) per BMS
#define CMD_BUFFER_SIZE(datalen) (4 + (N_BMS * (datalen + 2)))

#define BUFFER_BMS_OFFSET(bms, datalen) (4 + (bms * (datalen + 2)))

#define CMD_EMPTY_BUFFER ((uint8_t[CMD_BUFFER_SIZE(0)]){0})
#define CMD_EMPTY_BUFFER_SIZE CMD_BUFFER_SIZE(0)

//macro to function for better type checking (access attribute)
[[gnu::access(read_only, 4, 1)]]
static inline void __modify_writeCMD_args(size_t arglen, uint8_t args[static restrict N_BMS * (arglen + 2)], uint8_t bms, uint8_t * restrict data) {
  for (uint8_t i = 0; i < arglen; i++) {
    args[(bms * (arglen + 2)) + i] = data[i];
  }
}

#define modify_writeCMD_args(args, bms, data) \
  static_assert(bms < N_BMS, "bms out of range"); \
  __modify_writeCMD_args(args.arglen, args.args + 4, bms, data)


HAL_StatusTypeDef ___writeCMD(uint16_t command, uint8_t * args, size_t arglen);

[[gnu::access(read_write, 2, 4), gnu::nonnull(2), gnu::always_inline]] //add dummy size variable for bounds checking, should be optimized out
static inline HAL_StatusTypeDef __writeCMD(uint16_t command, uint8_t * args, size_t arglen, size_t _) {
  return ___writeCMD(command, args, arglen);
}

#define writeCMD(command, args, arglen) \
  __writeCMD(command, args, arglen, CMD_BUFFER_SIZE(arglen))

[[gnu::access(read_write, 2, 3), gnu::nonnull(2)]]
HAL_StatusTypeDef readCMD(uint16_t command, uint8_t * buffer, size_t buflen);

uint8_t pollCMD(uint16_t command);


void mcuAdbmsCSLow();
void mcuAdbmsCSHigh();

uint8_t wakeUpCmd();
static inline void mcuDelay(uint16_t delay) { HAL_Delay(delay); };

#endif /* ADBMS_LL_DRIVER_H_ */
